C  ****************************************************************************
C  *                                                                          *
C  *                          *****     CTF.F     *****                       *
C  *                                                                          *
C  *    LIBRARY OF MISCELLANEOUS CTF SUBROUTINES/FUNCTIONS:                   *
C  *                                                                          *
C  *     CTF          - Compute value of CTF at particular spatial frequency. *
C  *     CTF_ASTIG    - Compute defocus value for astigmatic data.            *
C  *     CTF_CURVE    - Compute complete CTF curve (as a table).              *
C  *     CTF_DIVIDE   - Multiply image FFT by inverse CTF.                    *
C  *     CTF_MULTIPLY - Multiply image FFT by CTF.                            *
C  *     CTF_NODES    - Find positions of nodes in general CTF.               *
C  *     PCTF_NODE    - Find positions of nodes in phase CTF.                 *
C  ****************************************************************************
C
C
C  ****************************************************************************
C  *                                                                          *
C  *                      *****     FUNCTION CTF     *****                    *
C  *                                                                          *
C  *    COMPUTE VALUE OF CTF AT PARTICULAR SPATIAL FREQUENCY, TAKING INTO     *
C  *     ACCOUNT THE DECAY EFFECTS OF SPATIAL AND TEMPORAL COHERENCE.         *
C  *                                                                          *
C  *    The basic formula used here is as described by Bottcher et al.        *
C  *     (Structure 4:387-394[1996]) which is:                                *
C  *                                                                          *
C  *       CTF = -[sqrt(1 - amp_fac**2)*sin(chi) +  amp_fac*cos(chi)]         *
C  *                                                                          *
C  *    To account for spatial and temporal coherence the CTF is then         *
C  *      multiplied by two separate exponential decay functions, Es and Et:  *
C  *                                                                          *
C  *                            CTF = CTF * Es * Et                           *
C  *                                                                          *
C  *    The definitions for Es and Et are found in the appropriate FUNCTION   *
C  *      routines, CTF_Es and CTF_Et.                                        *
C  *                                                                          *
C  *    Descriptins of the variables used to compute CTF are given in the     *
C  *     FUNCTION routines, CTF, CTF_Es, and CTF_Et.                          *
C  *                                                                          *
C  *    Calculations are performed after variables are converted to nm units. *
C  *       AMP_FAC is the fractional amplitude contrast.                      *
C  *       Cs is the spherical aberration coefficient (in mm)                 *
C  *       Cc is the chromatic aberration coefficient (in mm)                 *
C  *       dF is the defocus (in microns; positive for underfocus)            *
C  *       WAVELENGTH is the electron wavelength (in nm)                      *
C  *       D_STAR is the spatial frequency (in nm-1)                          *
C  *       BETA is the illumination aperture (in milliradians)                *
C  *                                                                          *
C  *    AUGUST 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.         *
C  *    LAST MODIFIED  1-SEP-1999 BY TSB.                                     * 
C  *    LAST MODIFIED  1-SEP-1999 BY TSB.                                     * 
C  ****************************************************************************
C
      FUNCTION CTF (D_STAR, PI, WAVELENGTH, dF, AMP_FAC, Cs, Cc, BETA, 
     1 dE, VOLTS)
C
      IMPLICIT NONE
C
      REAL*4 CTF, PI, WAVELENGTH, Cs, dF, D_STAR, AMP_FAC
      REAL*4 BETA, Cc, CTF_Es, Es, CTF_Et, Et, dE, VOLTS
      REAL*8 D_CTF, D_STAR_SQ, CHI, PART1, PART2, D_D_STAR
C  ----------------------------------------------------------------------------
C
      Es        = 1.0                  ! Initialize spatial coherence decay
      Et        = 1.0                  ! Initialize temporal coherence decay
      D_D_STAR  = D_STAR
      D_STAR_SQ = D_D_STAR * D_D_STAR
      PART1     = ((dF*1000.) * D_STAR_SQ) / 2.0
      PART2     = ((Cs*1000000.) * WAVELENGTH**2 * 
     1             D_STAR_SQ*D_STAR_SQ) / 4.0
      CHI       = 2.0 * PI * WAVELENGTH * (PART1 - PART2)
      D_CTF     = -( SQRT(1.0-AMP_FAC**2) * DSIN(CHI) + 
     1               AMP_FAC * DCOS(CHI) )
C
      IF(BETA.GT.0.0) THEN
        Es = CTF_Es (dF, BETA, D_STAR, WAVELENGTH, Cs)
      ENDIF
      IF(Cc.GT.0.0 .OR. dE.GT.0.0) THEN
        Et = CTF_Et (Cc, dE, VOLTS, WAVELENGTH, D_STAR)
      ENDIF
C
      CTF = D_CTF * Es * Et
CTSB
CTSB      WRITE(*,'(4F10.4)') D_STAR, CTF, Es, Et
CTSB
C      
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                 *****     SUBROUTINE CTF_ASTIG     *****                 *
C  *                                                                          *
C  *    COMPUTE DEFOCUS VALUE FOR ASTIGMATIC DATA FOR A PARTICULAR DIRECTION  *
C  *     WITH RESPECT TO THE X,Y IMAGE PLANE.                                 *
C  *                                                                          *
C  *    NOTE: FOR NON-ASTIGMATIC IMAGES (I.E. FOCUS_MAJ = FOCUS_MIN), AT EACH *
C  *          RADIUS OF THE IMAGE FFT, THE DEFOCUS REMAINS CONSTANT.          *
C  *                                                                          *
C  *    FOCUS_MAJ = DEFOCUS VALUE (IN NMS) ALONG MAJOR AXIS.                  *
C  *    FOCUS_MIN = DEFOCUS VALUE (IN NMS) ALONG MINOR AXIS.                  *
C  *    PHI       = ANGLE (IN RADIANS) SPECIFYING THE DIRECTION OF FOCUS_MAJ  *
C  *                 RELATIVE TO THE X-AXIS OF THE DIGITIZED IMAGE.           *
C  *    PSI       = ANGLE (IN RADIANS) OF A PARTICULAR FOURIER COMPONENT      *
C  *                 WHOSE DEFOCUS VALUE IS TO BE CALCULATED.                 *
C  *                                                                          *
C  *    AUGUST 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.         *
C  *    LAST MODIFIED  6-AUG-1999 BY T. BAKER                                 * 
C  ****************************************************************************
C
      SUBROUTINE CTF_ASTIG (FOCUS, FOCUS_MAJ, FOCUS_MIN, PSI, PHI)
C
      IMPLICIT NONE
C
      REAL*4    FOCUS, FOCUS_MAJ, FOCUS_MIN, PSI, PHI, ANG
C  ----------------------------------------------------------------------------
C
      IF(FOCUS_MAJ.EQ.FOCUS_MIN) THEN
         FOCUS = FOCUS_MAJ
       ELSE
         ANG   = PSI - PHI                                 ! IN RADIANS
         FOCUS = FOCUS_MAJ * COS(ANG)**2 + FOCUS_MIN * SIN(ANG)**2
      ENDIF
C      
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE CTF_DIVIDE     *****               *
C  *                                                                          *
C  *    DIVIDE IMAGE FFT BY CTF.                                              *
C  *                                                                          *
C  *    THE FORMULA USED depends on the value of FILTER:                      *
C  *                                                                          *
C  *    FILTER   FORMULA
C  *    --------------------------------------------------------------------  *
C  *      1      FILTER 4 (before first peak)+ FILTER 2 (after first peak)    *
C  *      2      FFTnew = (FFTold*ABS(CTF)*exp(TFAC))/(CTF**2 +               *    
C  *                                   FILT_FAC*(1-ABS(CTF))                  *
C  *      3      FFTnew = (FFTold*ABS(CTF)*exp(TFAC))/(CTF**2 + FILT_FAC)     *
C  *      4      FFTnew = (FFTold*exp(TFAC))/(ABS(CTF) + FILT_FAC*(1-ABS(CTF)) *
C  *      5      FFTnew = (FFTold*exp(TFAC))/(ABS(CTF) + FILT_FAC)            *
C  *                                                                          *
C  *    FILT_FAC = FACTOR TO MAKE SURE NOISE IS NOT MAGNIFIED    *
C  *               TOO MUCH NEAR THE NODES OF THE CTF.                        *
C  *    TFAC = TEMPERATURE FACTOR, SPECIFIED IN ANGSTROMS SQUARED, IS      *
C  *               USED TO SHARPEN UP HIGHER RESOUTION FEATURES.  VALUES      *
C  *               FOR CRYOTEM DATA TYPICALLY RANGE BETWEEN 100-500 A**2.     *
C  *               THIS TEMPERATURE FACTOR IS APPLIED AS AN INVERSE T_FAC.    *
C  *    CTFMODE = 0 DO NOTHING!                                               *
C  *            = 1 CORRECT PHASES AND AMPLITUDES                             *
C  *            = 2 FLIP PHASES WHERE NECESSARY; LEAVE AMPLITUDES UNCHANGED   *
C  *            = 3 ADJUST AMPLITUDES; LEAVE PHASES UNCHANGED                 *
C  *                                                                          *
C  *    AUGUST 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.         *
C  *    MODIFIED 12-AUG-1999 BY TSB (Fixed XTEST,YTEST problems).             *
C  *    LAST MODIFIED 20-JUN-2002 BY TSB/XZ/RWA (Implementing multiple filters).     *
C  ****************************************************************************

C
      SUBROUTINE CTF_DIVIDE (CTFMODE, FILTER, FFT, IDIM1, IDIM2, UNITS, 
     1 PIXSIZE, PI, RADIAN, WAVELENGTH, FOCUS_MAJ, FOCUS_MIN, ANG_MAJ, 
     2 AMP_FAC, Cs, FILT_FAC, TFAC, FFT_FULL, Cc, BETA, dE, VOLTS)
C
      IMPLICIT NONE
C
      INTEGER*4 CTFMODE, IDIM1, IDIM2, COUNT, IX, IY, IY2, FILTER
      INTEGER*4 UNITS, XTEST, YTEST, COUNT2
      REAL*4    FFT(1), YSQ, CTF, D_STAR, PIXSIZE, PSI, PHI, C, T
      REAL*4    FOCUS, FOCUS_MAJ, FOCUS_MIN, ANG_MAJ, TFAC
      REAL*4    PI, RADIAN, WAVELENGTH, AMP_FAC, Cs, X, Y, TMP, FACTOR
      REAL*4    Cc, BETA, dE, VOLTS, FILT_FAC(10), FFT_RAD, FFT_RAD_TEST
      LOGICAL   FFT_FULL, ASTIG
C  ----------------------------------------------------------------------------
C
      IF(CTFMODE.LE.0 .OR. CTFMODE.GT.3) RETURN
C
      PHI = ANG_MAJ * RADIAN            ! ANGLE OF FOCUS_MAJ WRT X-DIRECTION
C
      IF(UNITS.EQ.2) THEN
         TMP = 1.0/(IDIM1*PIXSIZE)                  ! ALREADY IN NM UNITS
       ELSE
         TMP = 10.0/(IDIM1*PIXSIZE)                  ! CONVERT TO NM UNITS
      ENDIF
C
      IF(FOCUS_MAJ.EQ.FOCUS_MIN) THEN
         ASTIG = .FALSE.
         FOCUS = FOCUS_MAJ
       ELSE
         ASTIG = .TRUE.
      ENDIF
C
      XTEST  = IDIM1/2
      YTEST  = IDIM2/2
      COUNT = -1
C
      IF(FFT_FULL) THEN
         IY2 = IDIM2
       ELSE
         IY2 = YTEST + 1
      ENDIF
C
      DO IY = 1,IY2                        ! PROCESS FFT A ROW AT A TIME
        IF(IY.GT.YTEST) THEN
           Y = FLOAT(IY-IDIM2-1)
         ELSE
           Y = FLOAT(IY-1)
        ENDIF
        YSQ = Y*Y
        DO IX = 1,IDIM1                  ! PROCESS CURRENT ROW
          COUNT  = COUNT + 2
          COUNT2 = COUNT + 1
          IF(IX.LE.XTEST) THEN
             X = FLOAT(IX-1)                  ! RIGHT HALF OF FFT
           ELSE
             X = FLOAT(IX-IDIM1-1)            ! LEFT HALF OF FFT
          ENDIF
          D_STAR = SQRT(YSQ + X*X) * TMP
          IF(ABS(X).GT.0.9) THEN            ! TRAP FOR FFT Y-AXIS DATA
             PSI = ATAN2 (Y,X)            ! ATAN2 GIVES ANGLE IN RADIANS
           ELSE
             IF(Y.GE.0.0) THEN
                PSI =  PI/2.0
              ELSE
                PSI = -PI/2.0
             ENDIF
          ENDIF
          IF(ASTIG) CALL CTF_ASTIG (FOCUS,FOCUS_MAJ,FOCUS_MIN,PSI,PHI)
          C = CTF (D_STAR, PI, WAVELENGTH, FOCUS, AMP_FAC, Cs, Cc, BETA, 
     1             dE, VOLTS)
          T = EXP ((TFAC/400.0) * D_STAR**2)                 ! ADJUSTED FOR NM
Ca          print *, T, tfac, d_star
C           ****************************
C-----------*  APPLY CORRECT FILTER    *
C           ****************************
          IF(FILTER.EQ.1) THEN
                CALL CTF_FIRSTPEAK_WEI (FOCUS, WAVELENGTH, FFT_RAD, 
     1          PIXSIZE, IDIM1,Cs, PI, units, amp_fac)
Ca              print *, 'focus,wave - ',focus,wavelength
Ca              print *, 'pixs,idim,cs,pi',pixSize,idim1,cs,pi
Ca              print *, 'fft_rad - ', fft_rad
              FFT_RAD_TEST = D_STAR / TMP
              IF (FFT_RAD_TEST .LE. FFT_RAD) THEN
                 FACTOR = T         /(ABS(C) + FILT_FAC(1)*(1.0-ABS(C)))
               ELSE
                 FACTOR = (T*abs(C))/(C*C    + FILT_FAC(2)*(1.0-abs(C)))
              ENDIF
C
             ELSE IF(FILTER.EQ.2) THEN
              FACTOR = (T*ABS(C))/(C*C + filt_fac(1)*(1.0-ABS(C))) ! "Misha+Wei"
             ELSE IF(FILTER.EQ.3) THEN
              FACTOR = (T*ABS(C))/(C*C + filt_fac(1))          ! "Misha" filter
             ELSE IF(FILTER.EQ.4) THEN
              FACTOR = T/(ABS(C) + filt_fac(1)*(1.0-ABS(C)))   ! "Wei" filter
             ELSE IF(FILTER.EQ.5) THEN
              FACTOR = T/(ABS(C) + filt_fac(1))                ! Original filter
          ENDIF
          IF(CTFMODE.EQ.1) THEN             ! ADJUST PHASES AND AMPLITUDES
             IF(C.GT.0.0) FACTOR = -FACTOR
             FFT(COUNT)  = FACTOR * FFT(COUNT)
             FFT(COUNT2) = FACTOR * FFT(COUNT2)
           ELSE IF(CTFMODE.EQ.2) THEN                     ! ADJUST PHASES ONLY
             IF(C.GT.0.0) THEN
               FFT(COUNT)  = -FFT(COUNT)
               FFT(COUNT2) = -FFT(COUNT2)
             ENDIF
           ELSE IF(CTFMODE.EQ.3) THEN                   ! ADJUST AMPLITUDES ONLY
             FFT(COUNT)  = FACTOR * FFT(COUNT)
             FFT(COUNT2) = FACTOR * FFT(COUNT2)
          ENDIF
        ENDDO
      ENDDO
C
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE CTF_DIVIDE_em3dr  *****             *
C  *                                                                          *
C  *    DIVIDE IMAGE FFT BY CTF.                                              *
C  *                                                                          *
C  *    THE FORMULA USED depends on the value of FILTER:                      *
C  *                                                                          *
C  *    FILTER   FORMULA
C  *    --------------------------------------------------------------------  *
C  *      1      FILTER 4 (before first peak)+ FILTER 2 (after first peak)    *
C  *      2      FFTnew = (FFTold*ABS(CTF)*exp(TFAC))/(CTF**2 +               *    
C  *                                   FILT_FAC*(1-ABS(CTF))                  *
C  *      3      FFTnew = (FFTold*ABS(CTF)*exp(TFAC))/(CTF**2 + FILT_FAC)     *
C  *      4      FFTnew = (FFTold*exp(TFAC))/(ABS(CTF) + FILT_FAC*(1-ABS(CTF)) *
C  *      5      FFTnew = (FFTold*exp(TFAC))/(ABS(CTF) + FILT_FAC)            *
C  *                                                                          *
C  *    FILT_FAC = FACTOR TO MAKE SURE NOISE IS NOT MAGNIFIED    *
C  *               TOO MUCH NEAR THE NODES OF THE CTF.                        *
C  *    TFAC = TEMPERATURE FACTOR, SPECIFIED IN ANGSTROMS SQUARED, IS      *
C  *               USED TO SHARPEN UP HIGHER RESOUTION FEATURES.  VALUES      *
C  *               FOR CRYOTEM DATA TYPICALLY RANGE BETWEEN 100-500 A**2.     *
C  *               THIS TEMPERATURE FACTOR IS APPLIED AS AN INVERSE T_FAC.    *
C  *    CTFMODE = 0 DO NOTHING!                                               *
C  *            = 1 CORRECT PHASES AND AMPLITUDES                             *
C  *            = 2 FLIP PHASES WHERE NECESSARY; LEAVE AMPLITUDES UNCHANGED   *
C  *            = 3 ADJUST AMPLITUDES; LEAVE PHASES UNCHANGED                 *
C  *                                                                          *
C  *   May 6, 2003 Xing/Rob - use this rtn in em3dr                           *
C  ****************************************************************************

C
      SUBROUTINE CTF_DIVIDE_em3dr (CTFMODE, FILTER, FFT, IDIM1, IDIM2, 
     1 UNITS, PIXSIZE, PI, RADIAN, WAVELENGTH, FOCUS_MAJ, FOCUS_MIN, 
     2 ANG_MAJ, AMP_FAC, Cs, FILT_FAC, TFAC, FFT_FULL, Cc, BETA, dE, 
     2 VOLTS, res_hi, res_out, minimum)
C
      IMPLICIT NONE
C
      INTEGER*4 CTFMODE, IDIM1, IDIM2, COUNT, IX, IY, IY2, FILTER
      INTEGER*4 UNITS, XTEST, YTEST, COUNT2
      REAL*4    FFT(1), YSQ, CTF, D_STAR, PIXSIZE, PSI, PHI, C, T
      REAL*4    FOCUS, FOCUS_MAJ, FOCUS_MIN, ANG_MAJ, TFAC
      REAL*4    PI, RADIAN, WAVELENGTH, AMP_FAC, Cs, X, Y, TMP, FACTOR
      REAL*4    Cc, BETA, dE, VOLTS, FILT_FAC(10), FFT_RAD, FFT_RAD_TEST
      real*4    res_hi, res_out, minimum, ds_hi, ds_out, t_max, weight
        real*4    delsq
      LOGICAL   FFT_FULL, ASTIG
C  ----------------------------------------------------------------------------
C
      IF(CTFMODE.LE.0 .OR. CTFMODE.GT.3) RETURN
C
      PHI = ANG_MAJ * RADIAN            ! ANGLE OF FOCUS_MAJ WRT X-DIRECTION
C
      IF(UNITS.EQ.2) THEN
         TMP = 1.0/(IDIM1*PIXSIZE)                  ! ALREADY IN NM UNITS
       ELSE
         TMP = 10.0/(IDIM1*PIXSIZE)                  ! CONVERT TO NM UNITS
      ENDIF
Ca  Xing:
      ds_hi = (idim1*pixsize) / res_hi
      ds_out = (idim1*pixsize) / res_out
      t_max = exp((tfac/400.0) * (ds_hi*tmp)**2)
      delsq = -(ds_out-ds_hi)**2 / log(minimum/t_max)
C
      IF(FOCUS_MAJ.EQ.FOCUS_MIN) THEN
         ASTIG = .FALSE.
         FOCUS = FOCUS_MAJ
       ELSE
         ASTIG = .TRUE.
      ENDIF
C
      XTEST  = IDIM1/2
      YTEST  = IDIM2/2
      COUNT = -1
C
      IF(FFT_FULL) THEN
         IY2 = IDIM2
       ELSE
         IY2 = YTEST + 1
      ENDIF
C
      DO IY = 1,IY2                        ! PROCESS FFT A ROW AT A TIME
        IF(IY.GT.YTEST) THEN
           Y = FLOAT(IY-IDIM2-1)
         ELSE
           Y = FLOAT(IY-1)
        ENDIF
        YSQ = Y*Y
        DO IX = 1,IDIM1                  ! PROCESS CURRENT ROW
          COUNT  = COUNT + 2
          COUNT2 = COUNT + 1
          IF(IX.LE.XTEST) THEN
             X = FLOAT(IX-1)                  ! RIGHT HALF OF FFT
           ELSE
             X = FLOAT(IX-IDIM1-1)            ! LEFT HALF OF FFT
          ENDIF
          D_STAR = SQRT(YSQ + X*X) * TMP
          IF(ABS(X).GT.0.9) THEN            ! TRAP FOR FFT Y-AXIS DATA
             PSI = ATAN2 (Y,X)            ! ATAN2 GIVES ANGLE IN RADIANS
           ELSE
             IF(Y.GE.0.0) THEN
                PSI =  PI/2.0
              ELSE
                PSI = -PI/2.0
             ENDIF
          ENDIF
          IF(ASTIG) CALL CTF_ASTIG (FOCUS,FOCUS_MAJ,FOCUS_MIN,PSI,PHI)
          C = CTF (D_STAR, PI, WAVELENGTH, FOCUS, AMP_FAC, Cs, Cc, BETA, 
     1             dE, VOLTS)
          T = EXP ((TFAC/400.0) * D_STAR**2)                 ! ADJUSTED FOR NM
Ca          print *, T, tfac, d_star
C           ****************************
C-----------*  APPLY CORRECT FILTER    *
C           ****************************
          IF(FILTER.EQ.1) THEN
                CALL CTF_FIRSTPEAK_WEI (FOCUS, WAVELENGTH, FFT_RAD, 
     1          PIXSIZE, IDIM1,Cs, PI, units, amp_fac)
Ca              print *, 'focus,wave - ',focus,wavelength
Ca              print *, 'pixs,idim,cs,pi',pixSize,idim1,cs,pi
Ca              print *, 'fft_rad - ', fft_rad
              FFT_RAD_TEST = D_STAR / TMP
              IF (FFT_RAD_TEST .LE. FFT_RAD) THEN
                 FACTOR = T         /(ABS(C) + FILT_FAC(1)*(1.0-ABS(C)))
               ELSE
                 FACTOR = (T*abs(C))/(C*C    + FILT_FAC(2)*(1.0-abs(C)))
              ENDIF
C
             ELSE IF(FILTER.EQ.2) THEN
              FACTOR = (T*ABS(C))/(C*C + filt_fac(1)*(1.0-ABS(C))) ! "Misha+Wei"
             ELSE IF(FILTER.EQ.3) THEN
              FACTOR = (T*ABS(C))/(C*C + filt_fac(1))          ! "Misha" filter
             ELSE IF(FILTER.EQ.4) THEN
              FACTOR = T/(ABS(C) + filt_fac(1)*(1.0-ABS(C)))   ! "Wei" filter
             ELSE IF(FILTER.EQ.5) THEN
              FACTOR = T/(ABS(C) + filt_fac(1))                ! Original filter
          ENDIF
          if (res_hi.gt.res_out) then
             if(fft_rad_test .le.ds_hi) then
              weight = 1.0
             else if (fft_rad_test.gt.ds_hi .and.
     1             fft_rad_test.le.ds_out) then
              weight = t_max * exp(-(fft_rad_test-ds_hi)**2/delsq)
     1              / t
             else if (fft_rad_test.gt.ds_out) then
              weight = 0.0
             endif
             factor = weight * factor
          endif

          IF(CTFMODE.EQ.1) THEN             ! ADJUST PHASES AND AMPLITUDES
             IF(C.GT.0.0) FACTOR = -FACTOR
             FFT(COUNT)  = FACTOR * FFT(COUNT)
             FFT(COUNT2) = FACTOR * FFT(COUNT2)
           ELSE IF(CTFMODE.EQ.2) THEN                     ! ADJUST PHASES ONLY
             IF(C.GT.0.0) THEN
               FFT(COUNT)  = -FFT(COUNT)
               FFT(COUNT2) = -FFT(COUNT2)
             ENDIF
           ELSE IF(CTFMODE.EQ.3) THEN                   ! ADJUST AMPLITUDES ONLY
             FFT(COUNT)  = FACTOR * FFT(COUNT)
             FFT(COUNT2) = FACTOR * FFT(COUNT2)
          ENDIF
        ENDDO
      ENDDO
C
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                *****     SUBROUTINE CTF_MULTIPLY     *****               *
C  *                                                                          *
C  *    MULTIPLY IMAGE FFT BY CTF.                                            *
C  *                                                                          *
C  *    PHI     = ANGLE (IN RADIANS) SPECIFYING THE DIRECTION OF FOCUS_MAJ    *
C  *               RELATIVE TO THE X-AXIS OF THE DIGITIZED IMAGE.             *
C  *    PSI     = ANGLE (IN RADIANS) OF A PARTICULAR FOURIER COMPONENT WHOSE  *
C  *               DEFOCUS VALUE IS TO BE CALCULATED.                         *
C  *    TEMPFAC = TEMPERATURE FACTOR, SPECIFIED IN ANGSTROMS SQUARED, IS      *
C  *               USED TO DAMPEN HIGH RESOUTION FEATURES.                    *
C  *                                                                          *
C  *    CTFMODE = 0 DO NOTHING!                                               *
C  *            = 1 MODIFY PHASES AND AMPLITUDES                              *
C  *            = 2 FLIP PHASES WHERE NECESSARY; LEAVE AMPLITUDES UNCHANGED   *
C  *            = 3 ADJUST AMPLITUDES; LEAVE PHASES UNCHANGED                 *
C  *                                                                          *
C  *    AUGUST 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.         *
C  *    MODIFIED 13-AUG-1999 BY TSB (Fixed XTEST,YTEST problems).             *
C  *    MODIFIED  2-MAY-2000 BY TSB (Added CTF_MODE switch).                  *
C  *    LAST MODIFIED  2-MAY-2000 BY TSB/RWA (Implement CTF decay).           *
C  ****************************************************************************
C
      SUBROUTINE CTF_MULTIPLY (CTFMODE, FFT, IDIM1, IDIM2, UNITS, 
     1 PIXSIZE, PI, RADIAN, WAVELENGTH, FOCUS_MAJ, FOCUS_MIN, ANG_MAJ, 
     2 AMP_FAC, Cs, Cc, BETA, dE, VOLTS, TEMPFAC, FFT_FULL)
C
      IMPLICIT NONE
C
      INTEGER*4 IDIM1, IDIM2, COUNT, IX, IY, IY2, XTEST, YTEST
      INTEGER*4 UNITS, CTFMODE
      REAL*4    FFT(1), YSQ, C, CTF, D_STAR, PIXSIZE, TMP
      REAL*4    FOCUS_MAJ, FOCUS_MIN, ANG_MAJ, PSI, PHI, X, Y
      REAL*4    PI, RADIAN, WAVELENGTH, dF, AMP_FAC, Cs
      REAL*4    TEMPFAC, T, TWOPI, FACTOR, Cc, BETA, dE, VOLTS
      LOGICAL   FFT_FULL, ASTIG
C  ----------------------------------------------------------------------------
C
      IF(CTFMODE.LE.0 .OR. CTFMODE.GT.3) RETURN
C
      TWOPI = 2.0*PI
      PHI   = ANG_MAJ * RADIAN        ! ANGLE OF FOCUS_MAJ WRT X-DIRECTION
C
      IF(UNITS.EQ.1) THEN
         TMP = 10.0/(IDIM1*PIXSIZE)                   ! CONVERT TO NM UNITS
       ELSE
         TMP = 1.0/(IDIM1*PIXSIZE)            ! ALREADY IN NM OR PIXEL UNITS
      ENDIF
C
      IF(FOCUS_MAJ.EQ.FOCUS_MIN) THEN
         ASTIG = .FALSE.
         dF    = FOCUS_MAJ
       ELSE
         ASTIG = .TRUE.
      ENDIF
C
      XTEST  = IDIM1/2
      YTEST  = IDIM2/2
      COUNT = -1
C
      IF(FFT_FULL) THEN
         IY2 = IDIM2
       ELSE
         IY2 = YTEST + 1
      ENDIF
C
      DO IY = 1,IY2                        ! PROCESS FFT A ROW AT A TIME
        IF(IY.GT.YTEST) THEN
           Y = FLOAT(IY-IDIM2-1)
         ELSE
           Y = FLOAT(IY-1)
        ENDIF
        YSQ = Y**2
        DO IX = 1,IDIM1                  ! PROCESS CURRENT ROW
          COUNT = COUNT + 2
          IF(IX.LE.XTEST) THEN
             X = FLOAT(IX-1)                  ! RIGHT HALF OF FFT
           ELSE
             X = FLOAT(IX-IDIM1-1)            ! LEFT HALF OF FFT
          ENDIF
          D_STAR = SQRT(YSQ + X**2) * TMP
          IF(ABS(X).GT.0.9) THEN            ! TRAP FOR FFT Y-AXIS DATA
             PSI = ATAN2 (Y,X)            ! ATAN2 GIVES ANGLE IN RADIANS
           ELSE
             IF(Y.GE.0.0) THEN
                PSI =  PI/2.0
              ELSE
                PSI = -PI/2.0
             ENDIF
          ENDIF
          IF(ASTIG) CALL CTF_ASTIG (dF,FOCUS_MAJ,FOCUS_MIN,PSI,PHI)
          C = CTF (D_STAR, PI, WAVELENGTH, dF, AMP_FAC, Cs, Cc, BETA, 
     1             dE, VOLTS)
          T = EXP( -((TEMPFAC/400.0) * D_STAR**2) )            ! ADJUSTED FOR NM
          FACTOR = ABS(T*C)
          IF(CTFMODE.EQ.1) THEN             ! ADJUST PHASES AND AMPLITUDES
             IF(C.GT.0.0) FACTOR = -FACTOR
             FFT(COUNT)   = FACTOR * FFT(COUNT)
             FFT(COUNT+1) = FACTOR * FFT(COUNT+1)
           ELSE IF(CTFMODE.EQ.2) THEN                     ! ADJUST PHASES ONLY
             IF(C.GT.0.0) THEN
               FFT(COUNT)   = -FFT(COUNT)
               FFT(COUNT+1) = -FFT(COUNT+1)
             ENDIF
           ELSE IF(CTFMODE.EQ.3) THEN                   ! ADJUST AMPLITUDES ONLY
             FFT(COUNT)   = FACTOR * FFT(COUNT)
             FFT(COUNT+1) = FACTOR * FFT(COUNT+1)
          ENDIF
        ENDDO
      ENDDO
C
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                 *****     SUBROUTINE PCTF_NODE     *****                 *
C  *                                                                          *
C  *    COMPUTE POSITIONS OF NODES FOR GIVEN DEFOCUS VALUE.  NOTE THAT IN     *
C  *     THIS ROUTINE ONLY THE PHASE CTF IS CONSIDERED.                       *
C  *                                                                          *
C  *                                                                          *
C  *    D_STAR = SPATIAL FREQUENCY (IN NMS**-1) WHERE "NODE" OCCURS.          *
C  *                                                                          *
C  *    AUGUST 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.         *
C  *    LAST MODIFIED 17-AUG-1999 BY T. BAKER                                 * 
C  ****************************************************************************
C
      SUBROUTINE PCTF_NODE (D_STAR, PI, WAVELENGTH, FOCUS, Cs, NODE)
C
      IMPLICIT NONE
C
      INTEGER*4 NODE
      REAL*4    A, B, BSQ, C, FOURAC, PI, WAVELENGTH, Cs, FOCUS
      REAL*4    DSQ, D_STAR, DIF, DSQ1, DSQ2
C  ----------------------------------------------------------------------------
C
      A = Cs*1000000.0*WAVELENGTH**3*0.5
      B = FOCUS*1000.0*WAVELENGTH
      C = FLOAT(NODE)
      BSQ = B*B
      FOURAC = 4.0*A*C
      DIF = BSQ - FOURAC
      IF(Cs.EQ.0.0) THEN
        PRINT *, 'CAN NOT SOLVE FOR DEFOCUS BCAUSE Cs = 0.0'
        CALL TSBEND
      ENDIF
      IF(FOCUS.LT.0.0) THEN                              ! OVERFOCUS
         DIF = BSQ + FOURAC
         DSQ = (B + SQRT (DIF))/(2.0*A)
       ELSE                                          ! UNDERFOCUS
         IF(DIF.LT.0.0) THEN
            DIF = BSQ - 4.0*A*(C-1.0)
            DSQ = (B + SQRT (DIF))/(2.0*A)
          ELSE
            DSQ = (B - SQRT (DIF))/(2.0*A)
         ENDIF
      ENDIF
      D_STAR = SQRT (DSQ)
C      
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                 *****     SUBROUTINE CTF_NODES     *****                 *
C  *                                                                          *
C  *    COMPUTE POSITIONS OF NODES FOR GIVEN DEFOCUS VALUE.  NOTE THAT IN     *
C  *     THIS ROUTINE THE GENERAL CTF IS CONSIDERED (I.E. WITH PHASE and      *
C  *     AMPLITUDE TERMS CONSIDERED).                                         *
C  *                                                                          *
C  *    D_STAR = SPATIAL FREQUENCY (IN NMS**-1) WHERE "NODE" OCCURS.          *
C  *                                                                          *
C  *    AUGUST 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.         *
C  *    LAST MODIFIED 17-AUG-1999 BY T. BAKER                                 * 
C  ****************************************************************************
C
      SUBROUTINE CTF_NODES (PI, WAVELENGTH, FOCUS, Cs, AMP_FAC, NODES)
C
      IMPLICIT NONE
C
      INTEGER*4 NODES, NODE, CYCLE
      REAL*4    CTF, PI, WAVELENGTH, Cs, FOCUS, AMP_FAC
      REAL*4    D_STAR, CHI, PART1, PART2, STEP
      LOGICAL   NEGCTF
C  ----------------------------------------------------------------------------
C
      CYCLE  = 1
      STEP   = 0.001
C
      D_STAR = 0.001
      PART1  = ((FOCUS*1000.) * D_STAR**2) / 2.0
      PART2  = ((Cs*1000000.) * WAVELENGTH**2 * D_STAR**4) / 4.0
      CHI    = 2.0 * PI * WAVELENGTH * (PART1 - PART2)
      CTF    = -( SQRT(1.0-AMP_FAC**2) * SIN(CHI) + 
     1         AMP_FAC * COS(CHI) )
      NEGCTF = .TRUE.
      IF(CTF.GT.0.0) NEGCTF = .FALSE.
C
      NODE   = 1
      D_STAR = 0.000
  100      CONTINUE
      DO WHILE (D_STAR.LT.2.0)
  200        D_STAR = D_STAR + STEP
        PART1 = ((FOCUS*1000.) * D_STAR**2) / 2.0
        PART2 = ((Cs*1000000.) * WAVELENGTH**2 * D_STAR**4) / 4.0
        CHI   = 2.0 * PI * WAVELENGTH * (PART1 - PART2)
        CTF   = -( SQRT(1.0-AMP_FAC**2) * SIN(CHI) + 
     1          AMP_FAC * COS(CHI) )
        IF(NEGCTF .AND. CTF.GT.0.0) THEN
           IF(CYCLE.LT.4) THEN
             D_STAR = D_STAR - STEP
             STEP   = STEP/10.0
             CYCLE  = CYCLE + 1
             GO TO 200
           ENDIF
           WRITE(*,'(A,I2,A,F10.4)') ' Node ', NODE, 
     1     ' occurs at D_STAR = ', D_STAR
           NEGCTF = .NOT.NEGCTF
           NODE   = NODE + 1
           STEP   = 0.001
           CYCLE  = 1
           GO TO 100
         ELSE IF (.NOT.NEGCTF .AND. CTF.LT.0.0) THEN
           IF(CYCLE.LT.4) THEN
             D_STAR = D_STAR - STEP
             STEP   = STEP/10.0
             CYCLE  = CYCLE + 1
             GO TO 200
           ENDIF
           WRITE(*,'(A,I2,A,F10.4)') ' Node ', NODE, 
     1     ' occurs at D_STAR = ', D_STAR
           NEGCTF = .NOT.NEGCTF
           NODE   = NODE + 1
           STEP   = 0.001
           CYCLE  = 1
           GO TO 100
        ENDIF
      ENDDO
C      
      RETURN
      END

C  ********************************************************************
C  *                                                                  *
C  *             *****     SUBROUTINE CTF_FIRSTPEAK_WEI *****         *
C  *                                                                  *
C  *    Written by Wei Zhang.                                         *
C  ********************************************************************
C
      SUBROUTINE CTF_FIRSTPEAK_WEI (defocus, wavelength1, fft_rad,
     $     pixel_size,IDIM1, Cs, pi, units, amp_fac)
C
      implicit none
C
        integer*4 idim1, idim2, units
        real*4  ctf, defocus, wavelength, pixel_size,wavelength1
        real*4  fw, fw2, theta, fft_rad, A, B, C, Cs, pi,pixsize2
      real*4  amp_fac, angle
C
Ca      Calc angle
      angle = asin(amp_fac)
      wavelength = wavelength1 * 10.0    ! We want wavelength in Angstroms
      A       = Cs*10000000.0            ! Want to use Cs in Angstroms
      B       = -2.0*defocus*10000.0     ! 10000. converts defocus to Angstroms
Ca Orig Way      C       = wavelength*10.0          ! Want wavelength in Angstroms
Ca  Xing's way:
Ca OLDWAY      C       = wavelength
      C       = wavelength * (1.0 - (2.0*angle)/pi)
      fw2     = (-sqrt(B*B - 4.0*A*C) - B)/(2.0*A)
      fw      = sqrt(fw2)
C
      if(units.eq.2) then
         pixsize2 = pixel_size * 10.0      ! Convert from nm to Angstroms
      else
         pixsize2 = pixel_size       ! Already Angstroms
      endif

Ca ORIG       fft_rad = fw*(pixel_size*idim1)/wavelength
      fft_rad = fw*(pixsize2*idim1)/wavelength
C
        return
        end

